@@ -2,27 +2,27 @@
 	wildmidi.c
 
  	Midi Player using the WildMidi Midi Processing Library
- 
- 	Copyright (C)2001-2004 Chris Ison
- 
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
-	(at your option) any later version.
 
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
+    Copyright (C) Chris Ison 2001-2010
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software
-	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This file is part of WildMIDI.
 
-	Email: cisos@bigpond.net.au
-		 wildcode@users.sourceforge.net
- 
- 	$Id: wildmidi.c,v 1.21 2004/01/28 05:45:09 wildcode Exp $
+    WildMIDI is free software: you can redistribute and/or modify the player
+    under the terms of the GNU General Public License and you can redistribute
+    and/or modify the library under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation, either version 3 of
+    the licenses, or(at your option) any later version.
+
+    WildMIDI is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License and
+    the GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU General Public License and the
+    GNU Lesser General Public License along with WildMIDI.  If not,  see
+    <http://www.gnu.org/licenses/>.
+
+    Email: wildcode@users.sourceforge.net
 */
 
 #include "config.h"
@@ -77,7 +77,7 @@ struct _midi_test {
  * offset 18-21 (0x12-0x15) - track size
  * offset 25 (0x1A) = bank number
  * offset 28 (0x1D) = patch number
- */ 
+ */
 unsigned char midi_test_c_scale[] = {
 	0x4d, 0x54, 0x68, 0x64, 0x00, 0x00, 0x00, 0x06, // 0x00
 	0x00, 0x00, 0x00, 0x01, 0x00, 0x06, 0x4d, 0x54, // 0x08
@@ -182,7 +182,7 @@ int (*send_output) (char * output_data, int output_size);
 void (*close_output) ( void );
 int audio_fd;
 
-inline void
+static void
 shutdown_output ( void ) {
 	printf("Shutting Down Sound System\n");
 	if (audio_fd != -1)
@@ -196,13 +196,13 @@ shutdown_output ( void ) {
 static char wav_file[1024] = "\0";
 unsigned long int wav_size;
 
-int write_wav_output (char * output_data, int output_size);
-void close_wav_output ( void );
+static int write_wav_output (char * output_data, int output_size);
+static void close_wav_output ( void );
 
-int
+static int
 open_wav_output ( void ) {
 
-	char wav_hdr[] = { 0x52, 0x49, 0x46, 0x46, 0x00, 0x00, 0x00, 0x00,
+	unsigned char wav_hdr[] = { 0x52, 0x49, 0x46, 0x46, 0x00, 0x00, 0x00, 0x00,
 				0x57, 0x41, 0x56, 0x45, 0x66, 0x6D, 0x74, 0x20,
 				0x10, 0x00, 0x00, 0x00, 0x01, 0x00, 0x02, 0x00,
 				0x44, 0xAC, 0x00, 0x00, 0x10, 0xB1, 0x02, 0x00,
@@ -229,7 +229,7 @@ open_wav_output ( void ) {
 		wav_hdr[30] = (bytes_per_sec >> 16) & 0xFF;
 		wav_hdr[31] = (bytes_per_sec >> 24) & 0xFF;
 	}
-	
+
 	if (write(audio_fd, &wav_hdr, 44) < 0) {
 		printf("ERROR: Writing Header %s\n", strerror(errno));
 		shutdown_output();
@@ -242,7 +242,7 @@ open_wav_output ( void ) {
 	return 0;
 }
 
-int
+static int
 write_wav_output (char * output_data, int output_size) {
 	if (write(audio_fd, output_data, output_size) < 0) {
 		printf("ERROR: Writing Wav %s\n", strerror(errno));
@@ -254,7 +254,7 @@ write_wav_output (char * output_data, int output_size) {
 	return 0;
 }
 
-void
+static void
 close_wav_output ( void ) {
 	char wav_count[4];
 	if (audio_fd == -1)
@@ -265,7 +265,11 @@ close_wav_output ( void ) {
 	wav_count[2] = (wav_size >> 16) & 0xFF;
 	wav_count[3] = (wav_size >> 24) & 0xFF;
 	lseek(audio_fd,40,SEEK_SET);
-	write(audio_fd,&wav_count,4);
+	if (write(audio_fd,&wav_count,4) < 0) {
+        printf("ERROR: Writing Wav %s\n", strerror(errno));
+		shutdown_output();
+	}
+
 
 	wav_size += 36;
 	wav_count[0] = (wav_size) & 0xFF;
@@ -273,7 +277,10 @@ close_wav_output ( void ) {
 	wav_count[2] = (wav_size >> 16) & 0xFF;
 	wav_count[3] = (wav_size >> 24) & 0xFF;
 	lseek(audio_fd,4,SEEK_SET);
-	write(audio_fd,&wav_count,4);
+    if (write(audio_fd,&wav_count,4) < 0) {
+        printf("ERROR: Writing Wav %s\n", strerror(errno));
+		shutdown_output();
+	}
 
 	shutdown_output();
 }
@@ -282,9 +289,9 @@ close_wav_output ( void ) {
 #if (defined _WIN32) || (defined __CYGWIN__)
 
 HWAVEOUT hWaveOut;
-WAVEHDR header; 
+WAVEHDR header;
 unsigned long int mm_buffer_count;
-static CRITICAL_SECTION waveCriticalSection; 
+static CRITICAL_SECTION waveCriticalSection;
 
 int write_mm_output (char * output_data, int output_size);
 void close_mm_output ( void );
@@ -298,21 +305,21 @@ unsigned long int mm_current_block = 0;
 
 static void CALLBACK mmOutProc( HWAVEOUT hWaveOut, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2 ) {
 	int* freeBlockCounter = (int*)dwInstance;
-	if(uMsg != WOM_DONE) 
-		return; 
+	if(uMsg != WOM_DONE)
+		return;
 	EnterCriticalSection(&waveCriticalSection);
 	 (*freeBlockCounter)++;
-	LeaveCriticalSection(&waveCriticalSection); 
-} 
+	LeaveCriticalSection(&waveCriticalSection);
+}
 
 
-int
+static int
 open_mm_output ( void ) {
-	WAVEFORMATEX wfx; 
+	WAVEFORMATEX wfx;
 	char *mm_buffer;
 	int i;
 
-	InitializeCriticalSection(&waveCriticalSection); 
+	InitializeCriticalSection(&waveCriticalSection);
 
 	if((mm_buffer = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, ((MM_BLOCK_SIZE + sizeof(WAVEHDR)) * MM_BLOCK_COUNT))) == NULL)  {
 		printf("Memory allocation error\n");
@@ -322,32 +329,32 @@ open_mm_output ( void ) {
 	mm_blocks = (WAVEHDR*)mm_buffer;
 	mm_buffer += sizeof(WAVEHDR) * MM_BLOCK_COUNT;
 
-	for(i = 0; i < MM_BLOCK_COUNT; i++) { 
+	for(i = 0; i < MM_BLOCK_COUNT; i++) {
 		mm_blocks[i].dwBufferLength = MM_BLOCK_SIZE;
 		mm_blocks[i].lpData = mm_buffer;
 		mm_buffer += MM_BLOCK_SIZE;
-	} 
+	}
 
 	wfx.nSamplesPerSec = rate;
-	wfx.wBitsPerSample = 16; 
-	wfx.nChannels = 2; 
+	wfx.wBitsPerSample = 16;
+	wfx.nChannels = 2;
 	wfx.cbSize = 0;
 	wfx.wFormatTag = WAVE_FORMAT_PCM;
 	wfx.nBlockAlign = (wfx.wBitsPerSample >> 3) * wfx.nChannels;
 	wfx.nAvgBytesPerSec = wfx.nBlockAlign * wfx.nSamplesPerSec;
 
-	if(waveOutOpen( &hWaveOut, WAVE_MAPPER, &wfx, (DWORD_PTR)mmOutProc, (DWORD_PTR)&mm_free_blocks, CALLBACK_FUNCTION ) != MMSYSERR_NOERROR) { 
+	if(waveOutOpen( &hWaveOut, WAVE_MAPPER, &wfx, (DWORD_PTR)mmOutProc, (DWORD_PTR)&mm_free_blocks, CALLBACK_FUNCTION ) != MMSYSERR_NOERROR) {
 		printf("unable to open WAVE_MAPPER device\n");
 		return -1;
-	 } 
-	 
-	
+	 }
+
+
 	send_output = write_mm_output;
 	close_output = close_mm_output;
 	return 0;
 }
 
-int 
+static int
 write_mm_output (char * output_data, int output_size) {
 	WAVEHDR* current;
 	int free_size = 0;
@@ -355,7 +362,7 @@ write_mm_output (char * output_data, int output_size) {
 	current = &mm_blocks[mm_current_block];
 
 	while (output_size) {
-		if(current->dwFlags & WHDR_PREPARED) 
+		if(current->dwFlags & WHDR_PREPARED)
 			waveOutUnprepareHeader(hWaveOut, current, sizeof(WAVEHDR));
 		free_size = MM_BLOCK_SIZE - current->dwUser;
 		if (free_size > output_size)
@@ -386,7 +393,7 @@ write_mm_output (char * output_data, int output_size) {
 	return 0;
 }
 
-void
+static void
 close_mm_output ( void ) {
 	WAVEHDR* current;
 	int i, j;
@@ -406,113 +413,80 @@ void *buffer;
 int bps;
 int alsa_first_time = 1;
 static snd_pcm_t  *pcm;
-static snd_pcm_uframes_t alsa_period_size;
-static snd_pcm_channel_area_t *areas;
 
-int write_alsa_output (char * output_data, int output_size);
-void close_alsa_output ( void );
+static int write_alsa_output (char * output_data, int output_size);
+static void close_alsa_output ( void );
 
-int
+static int
 open_alsa_output(void) {
 	snd_pcm_hw_params_t     *hw;
 	snd_pcm_sw_params_t     *sw;
 	int err;
-	int alsa_buffer_time, bits_per_sample;
+	unsigned int alsa_buffer_time;
 	unsigned int alsa_period_time;
-	snd_pcm_uframes_t alsa_buffer_size;
-	
-	if (!pcmname)
-		pcmname = "default";
-		
+
+	if (!pcmname) {
+	    pcmname = malloc (8);
+        strcpy(pcmname,"default\0");
+	}
+
 	if ((err = snd_pcm_open (&pcm, pcmname, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
 		printf("Error: audio open error: %s\n", snd_strerror (-err));
 		return -1;
-	} 
-	
+	}
+
 	snd_pcm_hw_params_alloca (&hw);
-	
+
 	if ((err = snd_pcm_hw_params_any(pcm, hw)) < 0) {
 		printf("ERROR: No configuration available for playback: %s\n", snd_strerror(-err));
-		
+
 		return -1;
 	}
 
-	if ((err = snd_pcm_hw_params_set_access(pcm, hw, SND_PCM_ACCESS_MMAP_INTERLEAVED)) < 0) {
+	if ((err = snd_pcm_hw_params_set_access(pcm, hw, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
 		printf("Cannot set mmap'ed mode: %s.\n", snd_strerror(-err));
 		return -1;
 	}
-	
-	if (snd_pcm_hw_params_set_format (pcm, hw, SND_PCM_FORMAT_S16) < 0) {
+
+	if (snd_pcm_hw_params_set_format (pcm, hw, SND_PCM_FORMAT_S16_LE) < 0) {
 		printf("ALSA does not support 16bit signed audio for your soundcard\n");
 		close_alsa_output();
 		return -1;
 	}
-	
+
 	if (snd_pcm_hw_params_set_channels (pcm, hw, 2) < 0) {
 		printf("ALSA does not support stereo for your soundcard\n");
 		close_alsa_output();
 		return -1;
 	}
-#ifdef ALSA_NEW
+
 	if (snd_pcm_hw_params_set_rate_near(pcm, hw, &rate, 0) < 0) {
-#else
-	if (snd_pcm_hw_params_set_rate_near(pcm, hw, rate, 0) < 0) {
-#endif
 		printf("ALSA does not support %iHz for your soundcard\n",rate);
 		close_alsa_output();
 		return -1;
 	}
-	
+
 	alsa_buffer_time = 500000;
 	alsa_period_time = 50000;
-#ifdef ALSA_NEW	
+
 	if ((err = snd_pcm_hw_params_set_buffer_time_near(pcm, hw, &alsa_buffer_time, 0)) < 0)
-#else
-	if ((err = snd_pcm_hw_params_set_buffer_time_near(pcm, hw, alsa_buffer_time, 0)) < 0)
-#endif
 	{
 		printf("Set buffer time failed: %s.\n", snd_strerror(-err));
 		return -1;
 	}
 
-#ifdef ALSA_NEW
 	if ((err = snd_pcm_hw_params_set_period_time_near(pcm, hw, &alsa_period_time, 0)) < 0)
-#else
-	if ((err = snd_pcm_hw_params_set_period_time_near(pcm, hw, alsa_period_time, 0)) < 0)
-#endif
 	{
 		printf("Set period time failed: %s.\n", snd_strerror(-err));
 		return -1;
 	}
-				
+
 	if (snd_pcm_hw_params(pcm, hw) < 0)
 	{
 		printf("Unable to install hw params\n");
 		return -1;
 	}
 
-#ifdef ALSA_NEW
-	if ((err = snd_pcm_hw_params_get_buffer_size(hw, &alsa_buffer_size)) < 0)
-#else
-	if ((err = snd_pcm_hw_params_get_buffer_size(hw)) < 0)
-#endif
-	{
-		printf ("snd_pcm_hw_params_get_buffer_size() failed: %s\n", snd_strerror(-err));
-		return -1;
-	}
-#ifdef ALSA_NEW
-	if ((err = snd_pcm_hw_params_get_period_size(hw, &alsa_period_size, 0)) < 0)	
-#else
-	alsa_buffer_size = err;
-	if ((err = snd_pcm_hw_params_get_period_size(hw, 0)) < 0)
-#endif
-	{
-		printf ("snd_pcm_hw_params_get_period_size() failed: %s\n", snd_strerror(-err));
-		return -1;
-	}
-#ifndef ALSA_NEW
-	alsa_period_size = err;
-#endif	
 	snd_pcm_sw_params_alloca(&sw);
 	snd_pcm_sw_params_current(pcm, sw);
 	if (snd_pcm_sw_params(pcm, sw) < 0)
@@ -520,66 +494,34 @@ open_alsa_output(void) {
 		printf("Unable to install sw params\n");
 		return -1;
 	}
-	
-	bits_per_sample = snd_pcm_format_physical_width(SND_PCM_FORMAT_S16);
-	bps = (rate * bits_per_sample * 2) / 8000;
 
-	buffer = malloc(alsa_period_size * bits_per_sample / 8 * 2);
-	areas = malloc(2 * sizeof(snd_pcm_channel_area_t));
-
-	areas[0].addr = buffer;
-	areas[0].first = 0;
-	areas[0].step = 2 * bits_per_sample;
-	areas[1].addr = buffer;
-        areas[1].first = bits_per_sample;
-        areas[1].step = 2 * bits_per_sample;
-	
 	send_output = write_alsa_output;
 	close_output = close_alsa_output;
-	return 0;			
+	if (pcmname != NULL) {
+	    free (pcmname);
+	}
+	return 0;
 }
 
-int 
+static int
 write_alsa_output (char * output_data, int output_size) {
-	int cnt = 0, err;
-	snd_pcm_uframes_t offset, frames;
-	snd_pcm_sframes_t avail;
-	const snd_pcm_channel_area_t *chan_areas = areas;
-	
+	int err;
+	snd_pcm_uframes_t frames;
+
 	while (output_size > 0) {
-		avail = snd_pcm_avail_update(pcm);
-		if (avail == -EPIPE) {
-			if (snd_pcm_state(pcm) == SND_PCM_STATE_XRUN) {
-				if ((err = snd_pcm_prepare(pcm)) < 0)
-					printf("snd_pcm_prepare() failed.\n");
-				alsa_first_time = 1;
-			}
-		} else if (avail < 0) {
-			printf("snd_pcm_avail_update() failed: %s\n", snd_strerror(-avail));
-			avail = 0;
-		}
-		if (avail < alsa_period_size) {
-			usleep(500);
-			continue;
-		}
 		frames = snd_pcm_bytes_to_frames(pcm, output_size);
-		if ((err = snd_pcm_mmap_begin(pcm, &chan_areas, &offset, &frames)) < 0) {
-			printf("snd_pcm_mmap_begin() failed: %s\n", snd_strerror(-err));
-		}
-		cnt = snd_pcm_frames_to_bytes(pcm, frames);
-		memcpy((char*) chan_areas[0].addr + snd_pcm_frames_to_bytes(pcm, offset), output_data, cnt);
-		if ((err = snd_pcm_mmap_commit(pcm, offset, frames)) < 0) {
+        if ((err = snd_pcm_writei(pcm, output_data, frames)) < 0) {
 			if (snd_pcm_state(pcm) == SND_PCM_STATE_XRUN) {
 				if ((err = snd_pcm_prepare(pcm)) < 0)
 					printf("snd_pcm_prepare() failed.\n");
 				alsa_first_time = 1;
+				continue;
 			}
+			return err;
 		}
-		if (err != frames)
-			printf("snd_pcm_mmap_commit returned %d, expected %d\n", err, (int)frames);
-		
-		output_size -= cnt;
-		output_data += cnt;
+
+		output_size -= snd_pcm_frames_to_bytes(pcm, err);
+		output_data += snd_pcm_frames_to_bytes(pcm, err);
 		if (alsa_first_time) {
 			alsa_first_time = 0;
 			snd_pcm_start(pcm);
@@ -588,11 +530,9 @@ write_alsa_output (char * output_data, int output_size) {
 	return 0;
 }
 
-void 
+static void
 close_alsa_output ( void ) {
 	snd_pcm_close (pcm);
-	free(areas);
-	free(buffer);
 }
 
 #else
@@ -611,7 +551,7 @@ struct audio_buf_info info;
 int write_oss_output (char * output_data, int output_size);
 void close_oss_output ( void );
 
-int
+static int
 open_oss_output( void ) {
 	int caps, rc, tmp;
 	unsigned long int omode = O_RDWR;
@@ -619,11 +559,14 @@ open_oss_output( void ) {
 	unsigned long int mmflags = MAP_FILE | MAP_SHARED;
 	unsigned long int sz = sysconf (_SC_PAGESIZE);
 
-	if (!pcmname)
-		pcmname = "/dev/dsp";
+   	if (!pcmname) {
+	    pcmname = malloc (8);
+        strcpy(pcmname,"default\0");
+	}
+
 	if ((audio_fd = open(pcmname, omode)) < 0) {
 		printf("ERROR: Unable to open /dev/dsp (%s)\n",strerror(errno));
-		return -1;		
+		return -1;
 	}
 	if (ioctl (audio_fd, SNDCTL_DSP_RESET, 0) < 0) {
 		printf("ERROR: Unable to reset /dev/dsp\n");
@@ -646,7 +589,7 @@ open_oss_output( void ) {
 		return -1;
 	}
 	max_buffer = (info.fragstotal * info.fragsize + sz - 1) & ~(sz - 1);
-	
+
 	rc = AFMT_S16_LE;
 	if (ioctl (audio_fd, SNDCTL_DSP_SETFMT, &rc) < 0 ) {
 		printf("Can't set 16bit\n");
@@ -660,20 +603,20 @@ open_oss_output( void ) {
 		shutdown_output();
 		return -1;
 	}
-		
+
 	if (ioctl (audio_fd, SNDCTL_DSP_SPEED, &rate) < 0) {
 		printf("ERROR: /dev/dsp does not support %iHz output\n",rate);
 		shutdown_output();
 		return -1;
 	}
-			
+
 	buffer = (unsigned char *) mmap(NULL, max_buffer, mmmode, mmflags, audio_fd, 0);
 	if (buffer == MAP_FAILED) {
 		printf("couldn't mmap %s\n",strerror(errno));
 		shutdown_output();
 		return -1;
 	}
-	
+
 	tmp = 0;
 	if (ioctl (audio_fd, SNDCTL_DSP_SETTRIGGER, &tmp) < 0) {
 		printf("Couldn't toggle\n");
@@ -681,7 +624,7 @@ open_oss_output( void ) {
 		shutdown_output();
 		return -1;
 	}
-	
+
 	tmp = PCM_ENABLE_OUTPUT;
 	if (ioctl (audio_fd, SNDCTL_DSP_SETTRIGGER, &tmp) < 0) {
 		printf("Couldn't toggle\n");
@@ -695,7 +638,7 @@ open_oss_output( void ) {
 	return 0;
 }
 
-int
+static int
 write_oss_output(char * output_data, int output_size) {
 	struct count_info count;
 	int data_read = 0;
@@ -731,7 +674,7 @@ write_oss_output(char * output_data, int output_size) {
 	return 0;
 }
 
-void
+static void
 close_oss_output(void) {
 	shutdown_output();
 	if (buffer != NULL)
@@ -756,17 +699,17 @@ static struct option const long_options[] = {
 	{"master_volume",1,0,'m'},
 	{"config_file",1,0,'c'},
 	{"wavout",1,0,'o'},
-	{"linear_vol",0,0,'l'},
+	{"log_vol",0,0,'l'},
 	{"reverb",0,0,'b'},
-	{"midi_test",0,0,'t'},
+	{"test_midi",0,0,'t'},
 	{"test_bank",1,0,'k'},
 	{"test_patch",1,0,'p'},
-	{"expensive",0,0,'e'},
-	{"device",1,0,'d'},
+	{"enhanced_resample",0,0,'e'},
+	{"auddev",1,0,'d'},
 	{NULL,0,NULL,0}
 };
 
-void
+static void
 do_help (void) {
 	printf("  -v    --version        Display version\n");
 	printf("  -h    --help           This help.\n");
@@ -777,25 +720,29 @@ do_help (void) {
 	printf("Software Wavetable Options\n");
 	printf("  -o W  --wavout=W       Saves the output to W in wav format\n");
 	printf("                         at 44100Hz 16 bit stereo\n");
-	printf("  -l    --linear_vol     Use linear volume adjustments\n");
+	printf("  -l    --log_vol        Use log volume adjustments\n");
 	printf("  -r N  --rate=N         output at N samples per second\n");
 	printf("  -c P  --config_file=P  P is the path and filename to your timidity.cfg\n");
 	printf("                         Defaults to /etc/timidity.cfg\n\n");
 	printf(" -m V  --master_volume=V Sets the master volumes, default is 100\n");
 	printf("                         range is 0-127 with 127 being the loudest\n");
+    printf(" -b    --reverb          Enable final output reverb engine\n");
+
 }
 
-void
+static void
 do_version (void) {
-	printf("WildMidi %s Open Source Midi Sequencer\n",WILDMIDI_VERSION);
-	printf("Copyright (C) 2001-2004 Chris Ison wildcode@users.sourceforge.net\n\n");
+	printf("\nWildMidi %s Open Source Midi Sequencer\n",PACKAGE_VERSION);
+	printf("Copyright (C) Chris Ison 2001-2010 wildcode@users.sourceforge.net\n\n");
 	printf("WildMidi comes with ABSOLUTELY NO WARRANTY\n");
 	printf("This is free software, and you are welcome to redistribute it\n");
-	printf("under the terms and conditions of the GNU General Public License version 2.\n");
+	printf("under the terms and conditions of the GNU General Public License version 3.\n");
 	printf("For more information see COPYING\n\n");
+	printf("Report bugs to %s\n",PACKAGE_BUGREPORT);
+	printf("WildMIDI homepage at %s\n\n",PACKAGE_URL);
 }
 
-void
+static void
 do_syntax (void) {
 	printf("wildmidi [options] filename.mid\n\n");
 }
@@ -824,13 +771,15 @@ main (int argc, char **argv) {
 	unsigned char *test_data = NULL;
 	unsigned char test_bank = 0;
 	unsigned char test_patch = 0;
-	static char *spinner="|/-\\";
+	static char spinner[] ="|/-\\";
 	static int spinpoint = 0;
+	unsigned long int seek_to_sample = 0;
+    int inpause = 0;
 
 #ifndef _WIN32
 	int my_tty;
 	struct termios _tty;
-	tcflag_t _res_oflg = _tty.c_oflag; 
+	tcflag_t _res_oflg = _tty.c_oflag;
 	tcflag_t _res_lflg = _tty.c_lflag;
 
 #define raw() (_tty.c_lflag &= ~(ICANON | ICRNL | ISIG), \
@@ -856,11 +805,9 @@ main (int argc, char **argv) {
 			case 'r': // SoundCard Rate
 				rate = atoi(optarg);
 				break;
-#ifdef EPERIMENT_626
 			case 'b': // Reverb
 				mixer_options ^= WM_MO_REVERB;
 				break;
-#endif
 			case 'm': // Master Volume
 				master_volume = (unsigned char)atoi(optarg);
 				break;
@@ -875,11 +822,11 @@ main (int argc, char **argv) {
 				pcmname = malloc (strlen(optarg)+1);
 				strcpy(pcmname,optarg);
 				break;
-			case 'e': // Expensive Interpolation
-				mixer_options |= WM_MO_EXPENSIVE_INTERPOLATION;
+			case 'e': // Enhanced Resampling
+				mixer_options |= WM_MO_ENHANCED_RESAMPLING;
 				break;
-			case 'l': // linear volume
-				mixer_options |= WM_MO_LINEAR_VOLUME;
+			case 'l': // log volume
+				mixer_options |= WM_MO_LOG_VOLUME;
 				break;
 			case 't': // play test midis
 				test_midi = 1;
@@ -895,12 +842,12 @@ main (int argc, char **argv) {
 				return 0;
 		}
 	}
-	
+
 	if (!config_file) {
-		config_file = malloc(sizeof(TIMIDITY_CFG)+1);
-		strncpy (config_file, TIMIDITY_CFG, sizeof(TIMIDITY_CFG));
-		config_file[sizeof(TIMIDITY_CFG)] = '\0';
-	}	
+		config_file = malloc(sizeof(WILDMIDI_CFG)+1);
+		strncpy (config_file, WILDMIDI_CFG, sizeof(WILDMIDI_CFG));
+		config_file[sizeof(WILDMIDI_CFG)] = '\0';
+	}
 	if ((optind < argc) || (test_midi)) {
 		printf("Initializing Sound System\n");
 
@@ -922,8 +869,10 @@ main (int argc, char **argv) {
 			}
 		}
 		printf("Initializing %s\n\n", WildMidi_GetString(WM_GS_VERSION));
-		printf(" +  Volume up       e  Better resampling     n  Next Midi\n");
-		printf(" -  Volume down     l  Linear volume         q  Quit\n\n");
+		printf(" +  Volume up        e  Better resampling    n  Next Midi\n");
+		printf(" -  Volume down      l  Log volume           q  Quit\n");
+        printf(" ,  1sec Seek Back   r  Reverb               .  1sec Seek Forward\n");
+        printf("                     p  Pause On/Off\n\n");
 
 		if (WildMidi_Init (config_file, rate, mixer_options) == -1) {
 			return 0;
@@ -931,12 +880,13 @@ main (int argc, char **argv) {
 		WildMidi_MasterVolume(master_volume);
 
 		output_buffer = malloc(16384);
+
 		if (output_buffer == NULL) {
 			printf("Not enough ram, exiting\n");
 			WildMidi_Shutdown();
 			return 0;
 		}
-#ifndef _WIN32		
+#ifndef _WIN32
 		my_tty = fileno(stdin);
 		if (isatty(my_tty)) {
 			savetty();
@@ -944,7 +894,7 @@ main (int argc, char **argv) {
 			fcntl(0, F_SETFL, FNONBLOCK);
 		}
 #endif
-		
+
 		while ((optind < argc) || (test_midi)) {
 			if (!test_midi) {
 				char * real_file = strrchr(argv[optind], '/');
@@ -988,7 +938,7 @@ main (int argc, char **argv) {
 				continue;
 			}
 			mixer_options = wm_info->mixer_options;
-			WildMidi_LoadSamples(midi_ptr);
+			fprintf(stderr, "\r");
 			while (1) {
 				count_diff = wm_info->approx_total_samples - wm_info->current_sample;
 				if (count_diff == 0)
@@ -999,30 +949,38 @@ main (int argc, char **argv) {
 				if (_kbhit()) {
 					ch = _getch();
 					putch(ch);
-				}				
+				}
 #else
 				if (read(my_tty, &ch, 1) != 1)
-					ch = 0; 
+					ch = 0;
 #endif
 				if (ch) {
 					switch (ch) {
 						case 'l':
-							WildMidi_SetOption(midi_ptr, WM_MO_LINEAR_VOLUME, ((mixer_options & WM_MO_LINEAR_VOLUME) ^ WM_MO_LINEAR_VOLUME));
-							mixer_options ^= WM_MO_LINEAR_VOLUME;
+							WildMidi_SetOption(midi_ptr, WM_MO_LOG_VOLUME, ((mixer_options & WM_MO_LOG_VOLUME) ^ WM_MO_LOG_VOLUME));
+							mixer_options ^= WM_MO_LOG_VOLUME;
 							break;
-#ifdef EXPERIMENT_626
 						case 'r':
-							WildMidi_SetOption(midi_ptr, WM_MO_REVERB, ((mixer_options & WM_MO_REVERB) ^ WM_MO_REVERB));	
-							mixer_options ^= WM_MO_REVERB;	
+							WildMidi_SetOption(midi_ptr, WM_MO_REVERB, ((mixer_options & WM_MO_REVERB) ^ WM_MO_REVERB));
+							mixer_options ^= WM_MO_REVERB;
 							break;
-#endif
 						case 'e':
-							WildMidi_SetOption(midi_ptr, WM_MO_EXPENSIVE_INTERPOLATION, ((mixer_options & WM_MO_EXPENSIVE_INTERPOLATION) ^ WM_MO_EXPENSIVE_INTERPOLATION));	
-							mixer_options ^= WM_MO_EXPENSIVE_INTERPOLATION;	
+							WildMidi_SetOption(midi_ptr, WM_MO_ENHANCED_RESAMPLING, ((mixer_options & WM_MO_ENHANCED_RESAMPLING) ^ WM_MO_ENHANCED_RESAMPLING));
+							mixer_options ^= WM_MO_ENHANCED_RESAMPLING;
 							break;
 						case 'n':
 							goto NEXTMIDI;
-						case 'q':	
+						case 'p':
+                            if (inpause) {
+                                inpause = 0;
+                                fprintf(stderr, "       \r");
+                            } else {
+                                inpause = 1;
+                                fprintf(stderr, "Paused \r");
+                                continue;
+                            }
+                            break;
+						case 'q':
 							printf("\n");
 							WildMidi_Close(midi_ptr);
 							WildMidi_Shutdown();
@@ -1034,7 +992,7 @@ main (int argc, char **argv) {
 #endif
 							printf("\r\n");
 							exit (0);
-						case '-':	
+						case '-':
 							if (master_volume > 0) {
 								master_volume--;
 								WildMidi_MasterVolume(master_volume);
@@ -1046,28 +1004,85 @@ main (int argc, char **argv) {
 								WildMidi_MasterVolume(master_volume);
 							}
 							break;
+                        // seeking examples
+                        case ',': // fast seek backwards
+                            if (wm_info->current_sample < rate) {
+                                seek_to_sample = 0;
+                            } else {
+                                seek_to_sample = wm_info->current_sample - rate;
+                            }
+                            WildMidi_FastSeek(midi_ptr, &seek_to_sample);
+                            break;
+                        case '.': // fast seek forwards
+                            if ((wm_info->approx_total_samples - wm_info->current_sample) < rate) {
+                                seek_to_sample = wm_info->approx_total_samples;
+                            } else {
+                                seek_to_sample = wm_info->current_sample + rate;
+                            }
+                            WildMidi_FastSeek(midi_ptr, &seek_to_sample);
+                            break;
+                        default:
+                            break;
 					}
 				}
 
+				if (inpause) {
+                    wm_info = WildMidi_GetInfo(midi_ptr);
+                    perc_play =  (wm_info->current_sample * 100) / wm_info->approx_total_samples;
+                    pro_mins = wm_info->current_sample / (rate * 60);
+                    pro_secs = (wm_info->current_sample % (rate * 60)) / rate;
+                    {
+                        int mode_count = 0;
+                        if (mixer_options & WM_MO_LOG_VOLUME) {
+                            modes[mode_count++] = 'l';
+                        }
+                        if (mixer_options & WM_MO_REVERB) {
+                            modes[mode_count++] = 'r';
+                        }
+                        if (mixer_options & WM_MO_ENHANCED_RESAMPLING) {
+                            modes[mode_count++] = 'e';
+                        }
+                        if (mode_count !=3) {
+                            do {
+                                modes[mode_count++] = ' ';
+                            } while (mode_count != 3);
+                        }
+                        modes[3] = '\0';
+                    }
+                    fprintf(stderr, "        [Approx %2lum %2lus Total] [%s] [%3i] [%2lum %2lus Processed] [%2lu%%] 0  \r",
+                        apr_mins, apr_secs, modes, master_volume,
+                        pro_mins, pro_secs, perc_play);
+
+#ifdef _WIN32
+                    Sleep(5);
+#else
+                    usleep(5000);
+#endif
+                    continue;
+				}
+
 				if (count_diff < 4096) {
 					output_result = WildMidi_GetOutput (midi_ptr, output_buffer, (count_diff * 4));
 				} else {
 					output_result = WildMidi_GetOutput (midi_ptr, output_buffer, 16384);
 				}
 
+                if (output_result <= 0)
+                    break;
+
 				wm_info = WildMidi_GetInfo(midi_ptr);
 				perc_play =  (wm_info->current_sample * 100) / wm_info->approx_total_samples;
 				pro_mins = wm_info->current_sample / (rate * 60);
 				pro_secs = (wm_info->current_sample % (rate * 60)) / rate;
 				{
 					int mode_count = 0;
-					if (mixer_options & WM_MO_LINEAR_VOLUME) {
+					if (mixer_options & WM_MO_LOG_VOLUME) {
 						modes[mode_count++] = 'l';
 					}
 					if (mixer_options & WM_MO_REVERB) {
 						modes[mode_count++] = 'r';
 					}
-					if (mixer_options & WM_MO_EXPENSIVE_INTERPOLATION) {
+					if (mixer_options & WM_MO_ENHANCED_RESAMPLING) {
 						modes[mode_count++] = 'e';
 					}
 					if (mode_count !=3) {
@@ -1077,17 +1092,14 @@ main (int argc, char **argv) {
 					}
 					modes[3] = '\0';
 				}
-				fprintf(stderr, "\r");
-				fprintf(stderr, "\t [Approx %2lum %2lus Total] [%s] [%3i] [%2lum %2lus Processed] [%2lu%%] %c  ", 
-					apr_mins, apr_secs, modes, master_volume, 
+				fprintf(stderr, "        [Approx %2lum %2lus Total] [%s] [%3i] [%2lum %2lus Processed] [%2lu%%] %c  \r",
+					apr_mins, apr_secs, modes, master_volume,
 					pro_mins, pro_secs, perc_play, spinner[spinpoint++%4]);
 
-				if (output_result == 0)
-					break;
 				send_output (output_buffer, output_result);
 			}
 NEXTMIDI:
-			fprintf(stderr, "\n\r");	
+			fprintf(stderr, "\n\r");
 			if (WildMidi_Close(midi_ptr) == -1) {
 				printf ("oops\n");
 			}
@@ -1115,7 +1127,7 @@ NEXTMIDI:
 		do_syntax();
 		return 0;
 	}
-			
+
 	if (output_buffer != NULL)
 		free(output_buffer);
 	printf("\r");
